import json

from lxml import etree

from .util import open_if_required
from .parse import hocr_page_iterator, hocr_page_to_word_data_fast
from .text import hocr_get_xml_page_offsets, hocr_get_plaintext_page_offsets, \
        hocr_page_text_from_word_data, get_paragraph_hocr_words


def hocr_get_page_lookup_table(fd_or_path):
    """
    Create lookup table for a given hOCR document. This allows for quickly
    jumping to specific XML pages.

    Args:

    * fd_or_path: file descriptor or filepath to the hOCR file

    Returns:

    Lookup table (list of a list) with each list entry:

    * [text_start_byte, text_end_byte, xml_start_byte, xml_end_byte]
    """
    text_ranges = hocr_get_plaintext_page_offsets(fd_or_path)
    xml_ranges = hocr_get_xml_page_offsets(fd_or_path)

    if len(text_ranges) != len(xml_ranges):
        # Perhaps use something other than ValueError
        raise ValueError('text_ranges and xml_ranges do not match')

    res = []
    for text, xml in zip(text_ranges, xml_ranges):
        res.append((text[0], text[1], xml[0], xml[1]))

    return res

def hocr_lookup_by_plaintext_offset(page_lookup_data, pos_bytes_plain):
    """
    Get the lookup index and data for a page that corresponds to the plaintext
    offset as
    specified in pos_bytes_plain.

    Args:

    * page_lookup_data: Lookup table as returned by hocr_load_lookup_table or
      hocr_get_page_lookup_table.
    * pos_bytes_plain: Offset in plaintext of the hOCR file.
    """
    for idx, dat in enumerate(page_lookup_data):
        tstart, tend = dat[0:2]
        xstart, xend = dat[2:4]

        if tstart <= pos_bytes_plain < tend:
            return idx, dat

    return None, None


def hocr_lookup_page_by_dat(fp, dat):
    """
    Get the XML for a specific hOCR page that corresponds to the lookup
    data `dat`.

    Args:

    * fp: file pointer to hOCR file
    * `dat`: lookup table entry for the page
    """
    xstart, xend = dat[2:4]

    fp.seek(xstart)
    xml = fp.read(xend-xstart)
    root = etree.fromstring(xml)
    return root


def hocr_lookup_page_by_plaintext_offset(fp, page_lookup_data, pos_bytes_plain):
    """
    Get the XML for a specific hOCR page that corresponds to the plaintext
    offset as specified in pos_bytes_plain.

    Args:

    * fp: file pointer to hOCR file
    * page_lookup_data: Lookup table as returned by hocr_load_lookup_table or
      hocr_get_page_lookup_table.
    * pos_bytes_plain: Offset in plaintext of the hOCR file.
    """
    _, dat = hocr_lookup_by_plaintext_offset(page_lookup_data, pos_bytes_plain)
    return hocr_lookup_page_by_dat(fp, dat)


def hocr_load_lookup_table(path):
    """
    Load lookup table from JSON.

    Args:

    * fd_or_path: File to load from
    
    Returns:

    * Lookup table
    """
    fp = open_if_required(path)
    return json.loads(fp.read().decode('utf-8'))


def hocr_save_lookup_table(lookup_table, fd_or_path):
    """
    Save lookup table to JSON.

    Args:

    * lookup_table: Lookup table as returned by hocr_get_page_lookup_table
    * fd_or_path: File to save to
    """
    if isinstance(fd_or_path, str):
        fd_or_path = open(fd_or_path, 'w+')
    json.dump(lookup_table, fd_or_path)


def hocr_get_fts_text(fd_or_path):
    """
    Return text that can be ingested in a full text search engine like SOLR or
    Elastic.

    Args:

    * fd_or_path: File descriptor or path to hOCR file

    Returns:

    Repeatedly yields a tuple of (``str``, ``list of int``),
    page text and a list of word confidences on the page.
    """
    for page in hocr_page_iterator(fd_or_path):
        word_data = hocr_page_to_word_data_fast(page)
        page_text = hocr_page_text_from_word_data(word_data)

        confs = []
        for paragraph in word_data:
            confs += [x['confidence'] for x in get_paragraph_hocr_words(paragraph)]

        yield page_text, confs



import gzip
import io

from lxml import etree

#: Contains the HOCR schema
HOCR_SCHEMA = '{http://www.w3.org/1999/xhtml}'


def open_if_required(fd_or_path):
    """
    Opens a file if `fd_or_path` is a `str`, otherwise returns `fd_or_path`.
    If `fd_or_path` ends with `.gz`, uses `gzip.open`.
    """
    if isinstance(fd_or_path, str):
        if fd_or_path.endswith('.gz'):
            xml_file = gzip.open(fd_or_path, 'rb')
        else:
            xml_file = open(fd_or_path, 'rb')
    else:
        xml_file = fd_or_path

    return xml_file


def get_ocr_system(fd):
    """
    Read the ocr-system meta tag from a new file descriptor containing a hOCR
    document. If you want to use an existing file descriptor, ensure to seek to
    the start first.

    Args:

    * fd: Open file descriptor

    Return:

    * string of the content system or None if none is specified
    """
    header, footer = get_header_footer(fd)

    bio = io.BytesIO()
    bio.write(header + footer)
    bio.seek(0)

    parse = etree.iterparse(bio, tag=(HOCR_SCHEMA+'meta',), events=('end',))
    for (start_end, element) in parse:
        if element.attrib.get('name') == 'ocr-system':
            return element.attrib.get('content')

    return None


def get_header_footer(fd):
    """
    Extract the parts before and after the body elements from a given XML file

    Args:

    * fd: Open file descriptor

    Returns:

    * Tuple (header, footer)
    """
    s = ''
    tags = (HOCR_SCHEMA+'html', HOCR_SCHEMA+'head')
    doc = etree.iterparse(fd, tag=tags, events=('start', 'end'))
    html_elem = None
    head_elem = None

    for act, elem in doc:
        if elem.tag[-4:] == 'html' and act == 'start':
            html_elem = elem
            children = html_elem.getchildren()
            for child in children:
                if child.tag[-4:] == 'body':
                    chs = child.getchildren()
                    for c in chs:
                        child.remove(c)
                    # Remove body, we add an empty one
                    html_elem.remove(child)

        if elem.tag[-4:] == 'head' and act == 'end':
            head_elem = elem
            body_elem = etree.Element('body')

            html_elem.append(head_elem)
            html_elem.append(body_elem)

            s = etree.tostring(html_elem, pretty_print=True, method='xml',
                               encoding='UTF-8', xml_declaration=True)
            s = s.decode('utf-8')
            break

    comp = s.split('<body>')

    # XML-ho
    header = comp[0] + '<body>' + '\n'
    htmlidx = header.find('<html')
    doctype = '''<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">'''
    header = header[:htmlidx] + doctype + '\n' + header[htmlidx:]

    footer = comp[1].lstrip()

    return header.encode('utf-8'), footer.encode('utf-8')




import xml.parsers.expat

from .util import open_if_required
from .parse import hocr_page_to_word_data_fast, hocr_page_iterator


def hocr_paragraph_text(paragraph):
    """
    Reconstruct text that matches the FTS text from a hOCR paragraph.
    Returns a tuple, first item in the tuple is the text, the second is a
    boolean, indicating if this paragraph is to be merged into the next one, see
    hocr_paragraphs for more information.

    Args:

    * paragraph: hOCR paragraph as returned by hocr_paragraphs

    Returns:

    * Tuple of (`str`, `bool`), where the `str` is the paragraph data, and the
      boolean if this text continues is to be merged with the next paragraph.
    """
    word_confidences = []
    par_text = ''

    for line in paragraph['lines']:
        line_words = ''
        for word in line['words']:
            line_words += word['text'] + ' '

        # Encode
        line_words = line_words.encode('utf-8')
        #line_words = line_words.strip().encode('utf-8')

        # Write out
        if line_words:
            par_text += line_words.decode('utf-8')

    if par_text:
        # Strip last space
        par_text = par_text[:-1]

    return par_text


def hocr_page_text_from_word_data(word_data):
    """
    Extract text from a pre-parsed hOCR page

    Args:

    * word_data: as returned by ``hocr_page_to_word_data`` or
      ``hocr_page_to_word_data_fast``

    Returns: page contents (`str`)
    """
    text = ''

    for paragraph in word_data:
        par_text = hocr_paragraph_text(paragraph)

        # Newline is something we add, it is not part of the paragraph text
        par_text += '\n'

        text += par_text

    return text


def hocr_page_text(page):
    """
    Extract text from a hOCR XML page element.

    Args:

    * page: hOCR XML page element

    Returns: page contents (`str`)
    """
    word_data = hocr_page_to_word_data_fast(page)
    return hocr_page_text_from_word_data(word_data)


def get_paragraph_hocr_words(paragraph):
    """
    Find all the words in a hOCR paragraph.

    Args:

    * hOCR paragraph as returned by hocr_paragraphs.

    Returns a `list` of hocr words in a hocr paragraph.
    For this to be usable for matching purposes, only run this on merged hocr
    paragraphs as returned by hocr_paragraphs.
    """
    words = []
    for line in paragraph['lines']:
        for word in line['words']:
            words.append(word)

    return words


# Finds bytes where pages start, and the final body segment to denote the end of
# the last page.
class PageFinder:
    def __init__(self, current_parser):
        self.parser = current_parser
        self.parser.StartElementHandler = self.start_element
        self.parser.EndElementHandler = self.end_element

        self.page_bytes = []

    def start_element(self, name, attrs):
        if name == 'div' and 'class' in attrs and attrs['class'] == 'ocr_page':
            self.page_bytes.append(self.parser.CurrentByteIndex)

    def end_element(self, name):
        if name == 'body':
            self.page_bytes.append(self.parser.CurrentByteIndex)


def hocr_get_xml_page_offsets(fd_or_path):
    """
    Builds a list of start and end bytes for each ocr_page element in the XML
    file.  This can be used to construct a "lookup" table, together with
    hocr_get_plaintext_page_offsets.

    Args:

    * fd_or_path: hOCR file to operate on, or a path (str).

    Return a list of tuples (start_byte, end_byte) for each ocr_page element in
    a hOCR file. The start and ends bytes point to the position of the page
    element in the XML file.
    """
    xml_file = open_if_required(fd_or_path)
    xml_file.seek(0)

    p = xml.parsers.expat.ParserCreate()
    h = PageFinder(p)
    p.ParseFile(xml_file)

    page_boundaries = list(zip(h.page_bytes[:-1], h.page_bytes[1:]))

    return page_boundaries


def hocr_get_plaintext_page_offsets(fd_or_path):
    """
    Builds a list of start and end bytes for each ocr_page in the plain text
    file. That is, if the plain text were generated from a hOCR XML file, which
    plaintext is part of which ocr_page element, and where does that text start
    and end. This can be used to construct a "lookup" table, together with
    hocr_get_xml_page_offsets.

    Args:

    * fd_or_path: hOCR file to operate on, or a path (str).

    Return a list of tuples (start_byte, end_byte) for each ocr_page element in
    a hOCR file. The start and ends bytes point to the position of the text as
    extracted from the page in the XML file.
    """
    page_it = hocr_page_iterator(fd_or_path)

    page_bytes = []
    cursor = 0

    page_bytes.append(0)

    for page in page_it:
        page_text = hocr_page_text(page)
        cursor += len(page_text)
        page_bytes.append(cursor)

    page_bytes = list(zip(page_bytes[:-1], page_bytes[1:]))

    return page_bytes



import gzip
import re

from lxml import etree

from .util import open_if_required, HOCR_SCHEMA


WRITING_DIRECTION_UNSPECIFIED = 0
WRITING_DIRECTION_LEFT_TO_RIGHT = 1
WRITING_DIRECTION_RIGHT_TO_LEFT = 2
WRITING_DIRECTION_TOP_TO_BOTTOM = 3

wdmap = {
    'ltr': WRITING_DIRECTION_LEFT_TO_RIGHT,
    'rtl': WRITING_DIRECTION_RIGHT_TO_LEFT,
}

BBOX_REGEX = re.compile(r'bbox((\s+-?\d+){4})')
PPI_REGEX = re.compile(r'scan_res((\s+\d+){2})')
BASELINE_REGEX = re.compile(r'baseline((\s+[\d\.\-]+){2})')
X_WCONF_REGEX = re.compile(r'x_wconf((\s+[\d\.\-]+){1})')
X_FSIZE_REGEX = re.compile(r'x_fsize((\s+[\d\.\-]+){1})')


def hocr_page_iterator(fd_or_path):
    """
    Returns an iterator to iterate over a (potentially large) hOCR XML file in a
    streaming manner.

    Args:

    * fd_or_path: open file to operate on, or a path (str).

    Returns:

    * Iterator returning a etree.Element hOCR page.
    """
    fp = open_if_required(fd_or_path)

    # Seek to start
    fp.seek(0)

    # TODO: Add gzip loading, specify what file_like should be (I suggest just
    # file descriptor or just path)
    doc = etree.iterparse(fp, tag=(HOCR_SCHEMA + 'div', 'div'))
    for act, elem in doc:
        if elem.tag[-3:] == 'div' and elem.attrib['class'] == 'ocr_page':
            page = elem
            yield page

            elem.clear()


def hocr_page_get_dimensions(hocr_page):
    """
    Returns the dimensions (width, height) of a hocr page as returned by
    hocr_page_iterator.

    Args:

    * hocr_page: a page as returned by hocr_page

    Returns:

    * (width, height): tuple of (int, int)
    """
    pagebox = BBOX_REGEX.search(hocr_page.attrib['title']).group(1).split()
    width, height = int(pagebox[2]), int(pagebox[3])
    return width, height


def hocr_page_get_scan_res(hocr_page):
    """
    Returns the X and Y resolution (in DPI) of a hocr page as returned by
    hocr_page_iterator.

    Args:

    * hocr_page: a page as returned by hocr_page

    Returns:

    * (x_res, y_res): tuple of (int, int)

    Or (None, None) if the scan_res property is not present.
    """
    pageppi = PPI_REGEX.search(hocr_page.attrib['title'])
    if pageppi:
        pageppi = pageppi.group(1).split()
        x_res, y_res = int(pageppi[0]), int(pageppi[1])
        return (x_res, y_res)
    else:
        return (None, None)


# XXX: Maybe get rid of scaler here, and just move the normalisation of the
# x_fsize to pdfrenderer.py
def hocr_page_to_word_data(hocr_page, scaler=1):
    """
    Parses a single hocr_page into word data.

    Args:

    * hocr_page: a single hocr_page as returned by hocr_page_iterator
    * (optional) scaler: a scalar to scale font sizes by

    Returns:

    A list of paragraphs, each paragraph containing a list of lines, and each
    line containing a list of words, plus properties.

    Paragraphs have the following attributes:

    * `'lines'`: the lines that form this paragraph

    Lines have the following attributes:

    * `'words'`: the words that form this line
    * `'bbox'`: bounding box (tuple of 4 floats)
    * `'baseline'`: baseline of the word (tuple of 2 floats)

    Words have the following attributes:

    * `'text'`: word text, str
    * `'bbox'`: bounding box (tuple of 4 floats)
    * `'fontsize'`: fontsize as a float, or 0.
    * `'writing_direction'`: See WRITING_DIRECTION_* constants
    * `'confidence'`: word confidence, 0 - 100
    """
    paragraphs = []

    for par in hocr_page.xpath('.//*[@class="ocrx_block" or @class="ocr_par"]'):
        paragraph_data = {'lines': []}

        paragraph_writing_direction = WRITING_DIRECTION_UNSPECIFIED
        if 'dir' in par.attrib:
            paragraph_writing_direction = wdmap[par.attrib['dir']]

        # We assume that the direct children are all the lines
        for line in par.getchildren():
            line_data = {}

            linebox = BBOX_REGEX.search(line.attrib['title']).group(1).split()
            baseline = BASELINE_REGEX.search(line.attrib['title'])
            if baseline is not None:
                baseline = baseline.group(1).split()
            else:
                baseline = [0, 0]

            linebox = [float(i) for i in linebox]
            baseline = [float(i) for i in baseline]

            line_data['bbox'] = linebox
            line_data['baseline'] = baseline

            word_data = []
            for word in line.xpath('.//*[@class="ocrx_word"]'):
                rawtext = ''
                wordbased = True
                for char in word.xpath('.//*[@class="ocrx_cinfo"]'):
                    rawtext += char.text
                    wordbased = False

                if wordbased:
                    wword = word
                    # Words may contains additional nodes like <em>
                    while True:
                        children = wword.getchildren()
                        if len(children) == 0:
                            break

                        if len(children) > 1:
                            raise ValueError('Not character based but word has multiple children?')

                        wword = children[0]

                    rawtext = wword.text

                    if wword.text is None:
                        raise ValueError('Word with no text value?')

                box = BBOX_REGEX.search(word.attrib['title']).group(1).split()
                box = [float(i) for i in box]

                conf = None
                m = X_WCONF_REGEX.search(word.attrib['title'])
                if m:
                    conf = int(m.group(1).split()[0])

                f_sizeraw = X_FSIZE_REGEX.search(word.attrib['title'])
                if f_sizeraw:
                    x_fsize = float(f_sizeraw.group(1))
                    x_fsize *= scaler
                else:
                    x_fsize = 0. # Will get fixed later on, in pdfrenderer at least

                writing_direction = WRITING_DIRECTION_UNSPECIFIED
                if 'dir' in word.attrib:
                    writing_direction = wdmap[word.attrib['dir']]
                else:
                    writing_direction = paragraph_writing_direction

                word_data.append({'bbox': box, 'text': rawtext, 'fontsize':
                    x_fsize, 'writing_direction': writing_direction,
                    'confidence': conf})


            line_data['words'] = word_data
            #print('Line words:', word_data)
            paragraph_data['lines'].append(line_data)

        paragraphs.append(paragraph_data)

    return paragraphs

def hocr_page_to_photo_data(hocr_page, minimum_page_area_pct=10):
    """
    Parses a single hocr_page into photo data.

    Args:

    * hocr_page: a single hocr_page as returned by hocr_page_iterator
    * (optional) minimum_page_area_pct: a minimum percentage of the page area the picture should inhabit

    Returns:

    A list of bounding boxes where photos were found
    """

    # Get the actual boxes from the page
    photo_boxes = []
    for photo in hocr_page.xpath('.//*[@class="ocr_photo"]'):
        box = BBOX_REGEX.search(photo.attrib['title']).group(1).split()
        box = [float(i) for i in box]
        photo_boxes.append(box)

    # Helper function to determine if there are nested boxes
    def box_contains_box(box_a, box_b):
        return box_a[0] <= box_b[0] and box_a[1] <= box_b[1] \
           and box_a[2] >= box_b[2] and box_a[3] >= box_b[3]

    # Clean up the box data a bit
    cleaned_photo_boxes = list(photo_boxes)
    dim = hocr_page_get_dimensions(hocr_page)
    area_page = dim[0]*dim[1]
    for box_a in photo_boxes:
        # Image must cover at least minimum_page_area_pct of page
        width, height = box_a[2]-box_a[0], box_a[3]-box_a[1]
        area_box = width*height
        if area_box < area_page*(minimum_page_area_pct/100.):
            try:
                cleaned_photo_boxes.remove(box_a)
                #print("Box %s is too small, removing" % (box_a))
            except: # Already removed
                pass

        # Nested boxes are redundant
        for box_b in photo_boxes:
            if box_a == box_b:
                continue
            if box_contains_box(box_a, box_b):
                try:
                    cleaned_photo_boxes.remove(box_b)
                    #print("Box %s is fully inside box %s, removing" % (box_b, box_a))
                except: # Already removed
                    pass

    return cleaned_photo_boxes

def get_title_attrs(title):
    # Assume Tesseract generated hOCR, where every ';' has a space after it
    sub_title = title.split('; ')
    box = None
    conf = None

    for subt in sub_title:
        if subt[0:7] == 'x_wconf':
            conf = int(subt[8:])
            continue
        if subt[0:4] == 'bbox':
            # TODO: use int()?
            #box = [float(i) for i in subt[i + 5:].split()]
            box = [int(i) for i in subt[5:].split()]
            continue

    return box, conf

def hocr_page_to_word_data_fast(hocr_page):
    """
    Parses a single hocr_page into word data.

    Args:

    * hocr_page: a single hocr_page as returned by hocr_page_iterator

    Returns:

    A list of paragraph, each paragraph containing a list of lines, and each
    line containing a list of words, plus properties.

    Paragraphs have the following attributes:

    * `'lines'`: the lines that form this paragraph

    Lines have the following attributes:

    * `'words'`: the words that form this line

    Words have the following attributes:

    * `'text'`: word text, str
    * `'bbox'`: bounding box (tuple of 4 floats)
    * `'confidence'`: word confidence, 0 - 100
    """
    paragraphs = []

    has_ocrx_cinfo = 0

    for par in hocr_page.xpath('.//*[@class="ocrx_block" or @class="ocr_par"]'):
        paragraph_data = {'lines': []}

        # We assume that the direct children are all the lines
        for line in par.getchildren():
            line_data = {}

            word_data = []
            for word in line.xpath('.//*[@class="ocrx_word"]'):
                title = word.attrib['title']

                box, conf = get_title_attrs(title)

                rawtext = ''
                wordbased = True
                if has_ocrx_cinfo < 2:
                    for char in word.xpath('.//*[@class="ocrx_cinfo"]'):
                        rawtext += char.text
                        wordbased = False
                        has_ocrx_cinfo = 1

                if has_ocrx_cinfo == 0:
                    has_ocrx_cinfo = 2

                if wordbased:
                    # Words may contains additional nodes like <em>
                    while True:
                        children = word.getchildren()
                        if len(children) == 0:
                            break

                        if len(children) > 1:
                            raise ValueError('Not character based but word has multiple children?')

                        word = children[0]

                    rawtext = word.text

                    if word.text is None:
                        raise ValueError('Word with no text value?')

                word_data.append({'bbox': box, 'text': rawtext,
                                  'confidence': conf})


            line_data['words'] = word_data
            paragraph_data['lines'].append(line_data)

        paragraphs.append(paragraph_data)

    return paragraphs



import pytest
import requests


@pytest.fixture(scope='session')
def sim_hocr_file(tmp_path_factory):
    hocr = tmp_path_factory.getbasetemp() / 'sim_english-illustrated-magazine_1884-12_2_15_chocr.html.gz'
    #hocr = tmp_path_factory.getbasetemp() / 'sim_english-illustrated-magazine_1884-12_2_15_hocr.html.gz'

    url = 'https://archive.org/download/sim_english-illustrated-magazine_1884-12_2_15/sim_english-illustrated-magazine_1884-12_2_15_chocr.html.gz'
    #url = 'https://archive.org/download/sim_english-illustrated-magazine_1884-12_2_15/sim_english-illustrated-magazine_1884-12_2_15_hocr.html.gz'

    r = requests.get(url)

    hocr.write_bytes(r.content)

    return hocr



import pytest
from subprocess import check_output
from os.path import dirname, join

import json

@pytest.mark.usefixtures('sim_hocr_file')

def test_all(sim_hocr_file):
    sim_hocr_file = str(sim_hocr_file)
    basedir = dirname(sim_hocr_file)

    lookup_table_file = join(basedir, 'table.json')
    sim_plaintext_file = join(basedir, 'plaintext.txt')
    sim_plaintext_hl_file = join(basedir, 'plaintext-hl.txt')

    lookup_table = check_output(['hocr-lookup-create', '-f', sim_hocr_file])
    with open(lookup_table_file, 'wb+') as f:
        f.write(lookup_table)

    plaintext = check_output(['hocr-text', '-f', sim_hocr_file])
    with open(sim_plaintext_file, 'wb+') as f:
        f.write(plaintext)

    plaintext_hl = check_output(['fts-text-annotate', '-f',
                                 sim_plaintext_file, '-p', 'English'])
    with open(sim_plaintext_hl_file, 'wb+') as f:
        f.write(plaintext_hl)

    search_matches = check_output(['fts-text-match', '--hocr',
                                   sim_hocr_file, '--annotated-text',
                                   sim_plaintext_hl_file, '--table',
                                   lookup_table_file])

    data = []
    for line in search_matches.decode('utf-8').split('\n'):
        line = line.strip()
        if not line:
            continue
        d = json.loads(line)
        data.append(d)

    known_data = json.load(open('tests/hocr_res.json', 'r+'))

    assert data == known_data



from setuptools import setup, find_packages
from distutils.util import convert_path

main_ns = {}
ver_path = convert_path('hocr/version.py')
with open(ver_path) as ver_file:
    exec(ver_file.read(), main_ns)

version = main_ns['__version__']
setup(name='archive-hocr-tools',
      version=version,
      packages=['hocr'],
      description='hOCR (streaming) parsers and writers',
      author='Merlijn Boris Wolf Wajer',
      author_email='merlijn@archive.org',
      url='https://github.com/internetarchive/archive-hocr-tools',
      download_url='https://github.com/internetarchive/archive-hocr-tools/archive/%s.tar.gz' % version,
      keywords=['hOCR', 'Internet Archive'],
      license='AGPL-3.0',
      scripts=['bin/hocr-combine-stream', 'bin/hocr-fold-chars',
               'bin/hocr-text', 'bin/fts-text-annotate',
               'bin/fts-text-match', 'bin/hocr-lookup-check',
               'bin/hocr-lookup-create', 'bin/hocr-lookup-reconstruct',
               'bin/hocr-text-paragraphs', 'bin/hocr-extract-page',
               'bin/abbyy-to-hocr', 'bin/hocr-split-pages',
               'bin/hocr-flatten-pages', 'bin/hocr-confidence-filter',
               'bin/hocr-to-epub', 'bin/pdf-to-hocr'],
      classifiers=[
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: GNU Affero General Public License v3',
          'Programming Language :: Python :: 3',
      ],
      python_requires='>=3.6',
      include_package_data=True,
      install_requires=['lxml==4.6.5'],
      extras_require={
          'epub': ['ebooklib==0.17.1', 'internetarchive-deriver-module==1.0.1', 'iso639==0.1.4'],
          'pdf': ['PyMuPDF==1.19.2', 'numpy==1.21.3'],
      },
      package_data={'hocr': ['data/*']})
